import os
import re
import time
from threading import Lock

import requests

from . import __parse_content

__TTL = 3600
__LAST_FETCHED = 0
__CONTENT_LOCK = Lock()
__CONTENT = None


def __fetch_manuf():
    global __LAST_FETCHED, __CONTENT
    response = requests.get(
        "https://raw.githubusercontent.com/kkrypt0nn/manuf/refs/heads/main/manuf.txt",
        timeout=5,
    )
    if response.status_code == 200:
        with __CONTENT_LOCK:
            __CONTENT = __parse_content(response.text)
            __LAST_FETCHED = time.time()
    else:
        raise Exception("Failed to fetch online manuf.txt")


try:
    __fetch_manuf()
except Exception:
    with open(
        os.path.join(os.path.dirname(__file__), "manuf.txt"), "r", encoding="utf-8"
    ) as f:
        __CONTENT = __parse_content(f.read())
        __LAST_FETCHED = time.time()


def lookup(mac: str) -> str:
    global __LAST_FETCHED
    if time.time() - __LAST_FETCHED > __TTL:
        __fetch_manuf()

    new_mac = mac.upper().replace("-", ":")

    if not re.match(r"^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$", new_mac):
        raise ValueError("Invalid MAC address")

    if new_mac in __CONTENT.data:
        return __CONTENT.data[new_mac]
    if res := __CONTENT.slash_28.get(f"{new_mac[:10]}0:00:00/28"):
        return res
    if res := __CONTENT.slash_36.get(f"{new_mac[:13]}0:00/36"):
        return res

    prefix = new_mac[:8]
    return next(
        (
            __CONTENT.data[key]
            for key in sorted(__CONTENT.data)
            if key.startswith(prefix)
        ),
        "unknown",
    )



import unittest

from pymanuf.online import lookup


class Tests(unittest.TestCase):
    def test_d_link(self):
        self.assertEqual(lookup("C4:A8:1D:73:D7:8C"), "D-Link International")

    def test_netgear(self):
        self.assertEqual(lookup("9C:D3:6D:9A:CA:81"), "Netgear")

    def test_shanghai_broadwan_communications(self):
        self.assertEqual(
            lookup("40:ED:98:6F:DB:AC"), "Shanghai Broadwan Communications Co.,Ltd"
        )

    def test_piranha_ems(self):
        self.assertEqual(lookup("70:B3:D5:8C:CD:BE"), "Piranha EMS Inc.")

    def test_ieee_registration_authority(self):
        self.assertEqual(lookup("3C:24:F0:F0:BE:CF"), "IEEE Registration Authority")

    def test_samsung_electronics(self):
        self.assertEqual(lookup("24:FC:E5:AD:BB:89"), "Samsung Electronics Co.,Ltd")

    def test_invalid_address(self):
        with self.assertRaises(ValueError):
            lookup("G4:FC:E5:AD:BB:89")


if __name__ == "__main__":
    unittest.main()
